// fft.cpp: implementation of the MyFFT class.
//  This is a sone what modified version of Takuya OOURA's
//     original radix 4 FFT package.
//Copyright(C) 1996-1998 Takuya OOURA
//    (email: ooura@mmm.t.u-tokyo.ac.jp).
//////////////////////////////////////////////////////////////////////
#include "stdafx.h"
#include <math.h>
#include "MyFFT.h"

//////////////////////////////////////////////////////////////////////
// Local Defines
//////////////////////////////////////////////////////////////////////
#define K_2PI (8.0*atan(1))

#define K_AMPMAX 32767.0	//maximum sin wave Pk for 16 bit input data
#define K_MAXDB 0.0			//specifies total range of FFT
#define K_MINDB -140.0

#define OVER_LIMIT 32000	//limit for detecting over ranging inputs


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
MyFFT::MyFFT()
{
	m_FFTinCntr=0;
	m_Overload = FALSE;
	m_Invert = FALSE;
	m_Complex = -1;
	m_AveSize = 1;
	m_LastAve = 1;
	m_LastFFTSize = 0;
	m_AveCount = 0;
	m_TotalCount = 0;
	m_FFTSize = 1024;
	pWorkArea = NULL;
	pSinCosTbl = NULL;
	pWindowTbl = NULL;
	pFFTAveBuf = NULL;
	pFFTInBuf = NULL;
	pFFTSumBuf = NULL;
	m_pTranslateTbl = NULL;
	m_dBCompensation = K_MAXDB;
	m_IQoffset = 0.0;

	RealWin=RealData=MagData=ImgData=0;
}

MyFFT::~MyFFT()
{							// free all resources
	FreeMemory();
}

void MyFFT::FreeMemory()
{
	if(pWorkArea)
	{
		delete pWorkArea;
		pWorkArea = NULL;
	}
	if(pSinCosTbl)
	{
		delete pSinCosTbl;
		pSinCosTbl = NULL;
	}
	if(pWindowTbl)
	{
		delete pWindowTbl;
		pWindowTbl = NULL;
	}
	if(pFFTAveBuf)
	{
		delete pFFTAveBuf;
		pFFTAveBuf = NULL;
	}
	if(pFFTSumBuf)
	{
		delete pFFTSumBuf;
		pFFTSumBuf = NULL;
	}
	if(pFFTInBuf)
	{
		delete pFFTInBuf;
		pFFTInBuf = NULL;
	}
	if(m_pTranslateTbl)
	{
		delete m_pTranslateTbl;
		m_pTranslateTbl = NULL;
	}
	
	if(RealData)	delete []RealData;	RealData = 0;
	if(RealWin)		delete []RealWin;	RealWin = 0;
	if(ImgData)		delete []ImgData;	ImgData = 0;
	if(MagData)		delete []MagData;	MagData = 0;
}

///////////////////////////////////////////////////////////////////
//FFT initialization and parameter setup function
///////////////////////////////////////////////////////////////////
void MyFFT::SetFFTParams(int sfrq, int snum, int ave, BOOL invert, double dBCompensation, double IQoffset,int SampleFreq, BOOL Complex)
{
	int i;
	if(snum==0)	return;
	
	m_BinMin = 0;		//force recalculation of plot variables
	m_BinMax = 0;
	m_StartFreq = 0;
	m_StopFreq = 0;
	m_PlotWidth = 0;
	m_FFTinCntr=0;
	m_Invert = invert;
	m_SampleFreq = (double)snum;
	//size=2048;

	//if changed real or complex then force
	if(m_Complex != Complex || m_dBCompensation != dBCompensation || m_AveSize != ave || m_IQoffset != IQoffset)
		{							
		//reset of FFT params
		m_LastFFTSize = 0;
		m_LastAve = 0;
		m_dBCompensation = dBCompensation;
		m_Complex = Complex;
		m_IQoffset = IQoffset;
		}

	if(ave>0) m_AveSize = ave;
	else m_AveSize = 1;

	
	if( (snum<=MAX_FFT_SIZE) && (snum>=MIN_FFT_SIZE) )	m_FFTSize = snum;
	else m_FFTSize = MAX_FFT_SIZE;
	if(m_LastFFTSize != m_FFTSize)
		{
		m_LastFFTSize = m_FFTSize;
		FreeMemory();
		pWindowTbl = new double[m_FFTSize];
		pSinCosTbl = new double[m_FFTSize];
		pWorkArea = new int[ (int)sqrt((double)m_FFTSize)+2];
		pFFTAveBuf = new double[m_FFTSize];
		pFFTSumBuf = new double[m_FFTSize];

		RealData	= new float[m_FFTSize];
		RealWin	= new float[m_FFTSize];
		ImgData	= new float[m_FFTSize];
		MagData	= new float[m_FFTSize];

		for(i=0; i<m_FFTSize; i++)
			{
			pFFTAveBuf[i] = 0.0;
			pFFTSumBuf[i] = 0.0;
			}
		pWorkArea[0] = 0;
		if(m_Complex)
			{
			pFFTInBuf = new double[m_FFTSize*2];
			m_pTranslateTbl = new int[m_FFTSize*2];
			for(i=0; i<m_FFTSize*2; i++) pFFTInBuf[i] = 0.0;
			makewt(m_FFTSize/2, pWorkArea, pSinCosTbl);
			}
		else
			{
			pFFTInBuf = new double[m_FFTSize];
			m_pTranslateTbl = new int[m_FFTSize];
			for(i=0; i<m_FFTSize; i++)
				pFFTInBuf[i] = 0.0;
			makewt(m_FFTSize/4, pWorkArea, pSinCosTbl);
			makect(m_FFTSize/4, pWorkArea, pSinCosTbl + pWorkArea[0]);
			}

//////////////////////////////////////////////////////////////////////
// A pure input sin wave ... Asin(wt)... will produce an fft output 
//   peak of (N*A/1)^2  where N is FFT_SIZE.
//		Kx = 2 for complex, 4 for real FFT
// To convert to a Power dB range:
//   PdBmax = 10*log10( (N*A/Kx)^2 + K_C ) + K_B
//   PdBmin = 10*log10( 0 + K_C ) + K_B
//  if (N*A/Kx)^2 >> K_C 
//  Then K_B = PdBmax - 20*log10( N*A/Kx )
//       K_C = 10 ^ ( (PdBmin-K_B)/10 )
//  for power range of 0 to 100 dB with input(A) of 32767 and N=262144
//			K_B = -86.63833  and K_C = 4.6114145e8
// To eliminate the multiply by 10, divide by 10 so for an output
//		range of 0 to -120dB the stored value is 0.0 to -12.0
//   so final constant K_B = -8.663833		
///////////////////////////////////////////////////////////////////////
		if(m_Complex) m_K_B = m_dBCompensation - 20*log10( (double)m_FFTSize*K_AMPMAX/2.0 );
		else m_K_B = m_dBCompensation - 20*log10( (double)m_FFTSize*K_AMPMAX/4.0 );
		m_K_C = pow( 10.0, (K_MINDB-m_K_B)/10.0 );
		m_K_B = m_K_B/10.0;
		for(i=0; i<m_FFTSize; i++)	pWindowTbl[i] = (.5  - .5 *cos( (K_2PI*i)/(m_FFTSize-1) ));	//Hanning
		ResetFFT();
		}
	if(m_LastAve != m_AveSize)
		{
		m_LastAve = m_AveSize;
		ResetFFT();
		}
}

///////////////////////////////////////////////////////////////////
//  Resets the FFT buffers and averaging variables.
///////////////////////////////////////////////////////////////////
void MyFFT::ResetFFT()
{
	
	for(int i=0; i<m_FFTSize;i++)
		{
		pFFTAveBuf[i] = 0.0;
		pFFTSumBuf[i] = 0.0;
		}
	m_AveCount = 0;
	m_TotalCount = 0;
	m_FFTinCntr=0;
}

//////////////////////////////////////////////////////////////////////
// "InBuf[]" is first multiplied by a window function, checked for overflow
//	and then placed in the FFT input buffers.  If the data is complex,
//  an IQoffset is also added to the input data.
//  The actual FFT is not performed here but is done if and when the 
//  user calls the GetxxxData(...) routines.  This is so if the user
//  cannot keep up with the input data, time is not wasted performing FFT's.
//	For real data there should be  m_FFTSize/2 InBuf data points
//	For complex data there should be  m_FFTSize InBuf data points
//////////////////////////////////////////////////////////////////////
int MyFFT::PutInFFT(short* InBuf)
{
	int i,j;
 	m_Overload = FALSE;
	
	if(m_Complex)
		{
		double dtmp1;
		double dtmp2 = m_IQoffset;
		j=0;
		for(i=0; i<m_FFTSize; i++)
			{
			dtmp1 = pWindowTbl[i];
			//NOTE: For some reason I and Q are swapped(demod I/Q does not apear to be swapped)
			pFFTInBuf[j] = dtmp1 * ((double)InBuf[j+1]+dtmp2);	//window the I data
			pFFTInBuf[j+1] = dtmp1 * ((double)InBuf[j]+dtmp2);	//window the Q data
			j += 2;
			}
		}
	else
		{
		for(i=0; i<m_FFTSize; i++)
			{
			pFFTInBuf[i] = pWindowTbl[i] * (double)InBuf[i];	//window the data
			}
		}
	
	return m_TotalCount;
}

void MyFFT::PutInFFT(float* InBuf, int size)
{
	int	i,max=m_FFTSize<<1;
	double dtmp1;
	double dtmp2 = m_IQoffset;
	if(m_Complex)
		{
		for(i=0; i<size; i+=2)
			{
			dtmp1 = pWindowTbl[m_FFTinCntr>>1];
			//NOTE: For some reason I and Q are swapped(demod I/Q does not apear to be swapped)
			pFFTInBuf[m_FFTinCntr] = dtmp1 * ((double)InBuf[i+1]+dtmp2);	//window the I data
			pFFTInBuf[m_FFTinCntr+1] = dtmp1 * ((double)InBuf[i]+dtmp2);	//window the Q data
			m_FFTinCntr += 2;
			if(m_FFTinCntr>=max) 
				m_FFTinCntr=0;
			}
		}
	else
		{
		for(i=0; i<size; i++)
			{
			pFFTInBuf[m_FFTinCntr] = pWindowTbl[m_FFTinCntr] * (double)InBuf[i];	//window the data
			if(++m_FFTinCntr>=m_FFTSize) m_FFTinCntr=0;
			}
		}
}

void MyFFT::PutInFFT(short* InBuf, int size)
{
	for(int i=0; i<size; i++)
		{
		pFFTInBuf[m_FFTinCntr] = pWindowTbl[m_FFTinCntr] * (double)InBuf[i];	//window the data
		if(++m_FFTinCntr>=m_FFTSize) m_FFTinCntr=0;
		}
}

//////////////////////////////////////////////////////////////////////
//The FFT is calculated and then the bin range is "start" to "stop".
// The range of start to stop frequencies are mapped to the users
// plot screen size so the users buffer will be filled with an array
// of integers whos value is the pixel height and the index of the
//  array is the x pixel coordinate.
// The function returns TRUE if the input is overloaded
//   This routine converts the data to 32 bit integers and is useful
//   when displaying fft data on the screen. 
//		MaxHeight = Plot height in pixels(zero is top and increases down)
//		MaxWidth =  Plot width in pixels
//		StartFreq = freq in Hz
//		StopFreq = freq in Hz
//		MaxdB = FFT dB level corresponding to output value == 0
//			must be <= to K_MAXDB
//		MindB = FFT dB level  corresponding to output value == MaxHeight
//			must be >= to K_MINDB
//////////////////////////////////////////////////////////////////////
BOOL MyFFT::GetScreenIntegerFFTData(float *InBuf, int MaxHeight,
								int MaxWidth,
								double MaxdB,
								double MindB,
								int StartFreq,
								int StopFreq,
								POINT *pt)
{
	int i,y,x,m;
	int ymax = 10000;
	int xprev = -1;
	int maxbin;
	double dBmaxOffset = MaxdB/10.0;
	double dBGainFactor = -10.0/(MaxdB-MindB);

	if(m_Complex)
		{
		StartFreq*=2;
		StopFreq*=2;
		StartFreq-=(int)m_SampleFreq/2;
		StopFreq-=(int)m_SampleFreq/2;
		}

	//if something has changed need to redo translate table
	if(m_StartFreq != StartFreq || m_StopFreq != StopFreq || m_LastComplex != m_Complex || m_PlotWidth != MaxWidth)
		{
		m_StartFreq = StartFreq;
		m_StopFreq = StopFreq;
		m_PlotWidth = MaxWidth;
		m_LastComplex = m_Complex;
		if(m_Complex)
			{	
			maxbin = m_FFTSize - 1;
			m_BinMin = (int)((double)StartFreq*(double)m_FFTSize/m_SampleFreq);
			m_BinMin += (m_FFTSize/2);
			m_BinMax = (int)((double)StopFreq*(double)m_FFTSize/m_SampleFreq);
			m_BinMax += (m_FFTSize/2);
			}
		else
			{
			maxbin = m_FFTSize/2 - 1;
			m_BinMin = (int)((double)StartFreq*(double)m_FFTSize/m_SampleFreq);
			m_BinMax = (int)((double)StopFreq*(double)m_FFTSize/m_SampleFreq);
			}

		//don't allow these go outside the translate table
		if(m_BinMin < 0) m_BinMin = 0;
		if(m_BinMin >= maxbin) m_BinMin = maxbin;
		if(m_BinMax < 0) m_BinMax = 0;
		if(m_BinMax >= maxbin) m_BinMax = maxbin;
		if( (m_BinMax-m_BinMin) > m_PlotWidth )
			{
			//if more FFT points than plot points
			for( i=m_BinMin; i<=m_BinMax; i++)
				m_pTranslateTbl[i] = ( (i-m_BinMin)*m_PlotWidth )/(m_BinMax - m_BinMin);
			}
		else
			{
			//if more plot points than FFT points
			for( i=0; i<m_PlotWidth; i++)
				m_pTranslateTbl[i] = m_BinMin + ( i*(m_BinMax - m_BinMin) )/m_PlotWidth;
			}
		}
/*
	if(m_Complex)
		{
		double dtmp1;
		double dtmp2 = m_IQoffset;
		for(j=i=0; i<m_FFTSize; i++)
			{
			dtmp1 = pWindowTbl[i];
			//NOTE: For some reason I and Q are swapped(demod I/Q does not apear to be swapped)
			pFFTInBuf[j] = dtmp1 * ((double)InBuf[j+1]+dtmp2);	//window the I data
			pFFTInBuf[j+1] = dtmp1 * ((double)InBuf[j]+dtmp2);	//window the Q data
			j += 2;
			}
		}
	else
		{
		for(i=0; i<m_FFTSize; i++)
			{
			pFFTInBuf[i] = pWindowTbl[i] * (double)InBuf[i];	//window the data
			}
		}*/
	
	if(m_Complex)
		{
		//Calculate the complex FFT
		bitrv2(m_FFTSize*2, pWorkArea + 2, pFFTInBuf);
		CpxFFT(m_FFTSize*2, pFFTInBuf, pSinCosTbl);
		m = (m_FFTSize);
		}
	else
		{
		//Calculate the real FFT
		bitrv2(m_FFTSize, pWorkArea + 2, pFFTInBuf);
		cftfsub(m_FFTSize, pFFTInBuf, pSinCosTbl);
		rftfsub(m_FFTSize, pFFTInBuf, pWorkArea[1], pSinCosTbl + pWorkArea[0]);
		m = (m_FFTSize/2);
		}

	if( (m_BinMax-m_BinMin) > m_PlotWidth )
		{
		//if more FFT points than plot points
		for( i=m_BinMin; i<=m_BinMax; i++ )
			{
			if(m_Complex)
				{
				if(m_Invert) y = (int)((double)MaxHeight*dBGainFactor*(pFFTAveBuf[(m-i)] - dBmaxOffset));
				else y = (int)((double)MaxHeight*dBGainFactor*(pFFTAveBuf[i] - dBmaxOffset));
				}
			else	//Is a real FFT
				{
				if(m_Invert) y = (int)((double)MaxHeight*dBGainFactor*(pFFTAveBuf[(m-i)<<1] - dBmaxOffset));
				else y = (int)((double)MaxHeight*dBGainFactor*(pFFTAveBuf[i<<1] - dBmaxOffset));
				y = (int)((double)MaxHeight*dBGainFactor*((double)MagData[i<<1] - dBmaxOffset));
				}
			if(y<0)	y = 0;
			if(y > MaxHeight) y = MaxHeight;
			x = m_pTranslateTbl[i];	//get fft bin to plot x coordinate transform
			if( x==xprev )	// still mappped to same fft bin coordinate
				{
				if(y < ymax)		//store only the max value
					{
					pt[x],y = y;
					ymax = y;
					}
				}
			else
				{
				pt[x].y = y;
				xprev = x;
				ymax = y;
				}
			}
		}
	else
		{
		//if more plot points than FFT points
		for( x=0; x<m_PlotWidth; x++ )
			{
			i = m_pTranslateTbl[x];	//get plot to fft bin coordinate transform
			if(m_Complex)
				{
				if(m_Invert) y = (int)((double)MaxHeight*dBGainFactor*(pFFTAveBuf[(m-i)] - dBmaxOffset));
				else y = (int)((double)MaxHeight*dBGainFactor*(pFFTAveBuf[i] - dBmaxOffset));
				}
			else	//Is a real FFT
				{
				if(m_Invert) y = (int)((double)MaxHeight*dBGainFactor*(pFFTAveBuf[(m-i)<<1] - dBmaxOffset));
				else y = (int)((double)MaxHeight*dBGainFactor*(pFFTAveBuf[i<<1] - dBmaxOffset));

				//y = (int)((double)MaxHeight*dBGainFactor*((double)MagData[i<<1] - dBmaxOffset));

				}
			if(y<0)	y = 0;
			if(y > MaxHeight) y = MaxHeight;
			pt[x].y = y;
			}
		}
	
	return m_Overload;
}


//////////////////////////////////////////////////////////////////////
//The FFT is calculated and then the bin range is "start" to "stop"
// and is copied into users double floating point buffer "OutBuf[]". 
// The function returns TRUE if the input is overloaded
//////////////////////////////////////////////////////////////////////
BOOL MyFFT::GetDoubleFFTData(int start, int stop, double* OutBuf )
{
	int m;
	if(m_Complex)
		{
		//Calculate the complex FFT
		bitrv2(m_FFTSize*2, pWorkArea + 2, pFFTInBuf);
		CpxFFT(m_FFTSize*2, pFFTInBuf, pSinCosTbl);
		m = (m_FFTSize);
		}
	else
		{
		//Calculate the real FFT
		bitrv2(m_FFTSize, pWorkArea + 2, pFFTInBuf);
		cftfsub(m_FFTSize, pFFTInBuf, pSinCosTbl);
		rftfsub(m_FFTSize, pFFTInBuf, pWorkArea[1], pSinCosTbl + pWorkArea[0]);
		m = (m_FFTSize/2);
		}
	if( (stop <= m) && (start<stop) )
		{
		
		for( int i=start; i<=stop; i++ )		//copy and scale into OutBuf[]
			{
			if(m_Complex)
				{
				if(m_Invert) OutBuf[i] = 10.0*pFFTAveBuf[(m-i)];
				else OutBuf[i] = 10.0*pFFTAveBuf[i];
				}
			else	//Is a real FFT
				{
				if(m_Invert) OutBuf[i] = 10.0*pFFTAveBuf[(m-i)<<1];
				else OutBuf[i] = 10.0*pFFTAveBuf[i<<1];
				}
			}
		
		}
	return m_Overload;
}

///////////////////////////////////////////////////////////////////
// Nitty gritty fft routines by Takuya OOURA(Updated to his new version 4-18-02)
// Routine calculates real FFT
///////////////////////////////////////////////////////////////////
void MyFFT::rftfsub(int n, double *a, int nc, double *c)
{
	double tmp1,tmp2;
    int j, k, kk, ks, m;
    double wkr, wki, xr, xi, yr, yi;

	m_TotalCount++;
 	if(m_AveCount < m_AveSize) m_AveCount++;
    m = n >> 1;
    ks = 2 * nc/m;
    kk = 0;
	for (j = 2; j < m; j += 2 ) 
		{
		k = n - j;
		kk += ks;
		wkr = 0.5 - c[nc - kk];
		wki = c[kk];
		xr = a[j] - a[k];
		xi = a[j + 1] + a[k + 1];
		yr = wkr * xr - wki * xi;
		yi = wkr * xi + wki * xr;
		a[j] -= yr;
		xi = a[j]*a[j];
		a[j+1] -= yi;
		xi += ( a[j+1]*a[j+1]);
		a[k] += yr;
		xr = a[k]*a[k];
		a[k+1] -= yi;
		xr += (a[k+1]*a[k+1]);
		tmp1 = log10(xi+m_K_C) + m_K_B;
		tmp2 = log10(xr+m_K_C) + m_K_B;
		if(m_TotalCount <= m_AveSize)
			{	//do straight average up until m_AveSize fft's
 			pFFTSumBuf[j] = pFFTSumBuf[j] + tmp1;
 			pFFTSumBuf[k] = pFFTSumBuf[k] + tmp2;
			}
		else
			{	//do IIR filter after m_AveSize fft's
 			pFFTSumBuf[j] = pFFTSumBuf[j] - pFFTAveBuf[j] + tmp1;
 			pFFTSumBuf[k] = pFFTSumBuf[k] - pFFTAveBuf[k] + tmp2;
			}
		pFFTAveBuf[j] = pFFTSumBuf[j]/(double)m_AveCount;
		pFFTAveBuf[k] = pFFTSumBuf[k]/(double)m_AveCount;
		}

	a[0] *= a[0];						//calc DC term
	xr = a[m]*a[m]+a[m+1]*a[m+1];		//calculate N/4(middle) term
	tmp1 = log10(a[0]+m_K_C) + m_K_B;
	tmp2 = log10(xr+m_K_C) + m_K_B;

	if(m_TotalCount <= m_AveSize)
		{	//do straight average up until m_AveSize fft's
 		pFFTSumBuf[0] = pFFTSumBuf[0] + tmp1;
 		pFFTSumBuf[n/2] = pFFTSumBuf[n/2] + tmp2;
		}
	else
		{	//do IIR filter after m_AveSize fft's
 		pFFTSumBuf[0] = pFFTSumBuf[0] - pFFTAveBuf[0] + tmp1;
 		pFFTSumBuf[n/2] = pFFTSumBuf[n/2] - pFFTAveBuf[n/2] + tmp2;
		}
	pFFTAveBuf[0] = pFFTSumBuf[0]/(double)m_AveCount;
	pFFTAveBuf[n/2] = pFFTSumBuf[n/2]/(double)m_AveCount;


	for (int i=1; i < m; i++)
		{
		//pFFTAveBuf[i] =(SQR(pFFTAveBuf[(2*i)+1]) + SQR(pFFTAveBuf[(2*i)+2])) ;
		}

}


///////////////////////////////////////////////////////////////////
// Routine calculates complex FFT
///////////////////////////////////////////////////////////////////
void MyFFT::CpxFFT(int n, double *a, double *w)
{
    int j, j1, j2, j3, l;
    double x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i;
    
	m_TotalCount++;
 	if(m_AveCount < m_AveSize)	m_AveCount++;
    l = 2;
    if (n > 8) 
		{
        cft1st(n, a, w);
        l = 8;
        while ((l << 2) < n) 
			{
            cftmdl(n, l, a, w);
            l <<= 2;
			}
		}
    if ((l << 2) == n) 
		{
        for (j = 0; j < l; j += 2)
			{
            j1 = j + l;
            j2 = j1 + l;
            j3 = j2 + l;
            x0r = a[j] + a[j1];
            x0i = a[j + 1] + a[j1 + 1];
            x1r = a[j] - a[j1];
            x1i = a[j + 1] - a[j1 + 1];
            x2r = a[j2] + a[j3];
            x2i = a[j2 + 1] + a[j3 + 1];
            x3r = a[j2] - a[j3];
            x3i = a[j2 + 1] - a[j3 + 1];
            a[j] = x0r + x2r;
            a[j + 1] = x0i + x2i;
            a[j2] = x0r - x2r;
            a[j2 + 1] = x0i - x2i;
            a[j1] = x1r - x3i;
            a[j1 + 1] = x1i + x3r;
            a[j3] = x1r + x3i;
            a[j3 + 1] = x1i - x3r;
			}
		}
	else 
		{
        for (j = 0; j < l; j += 2)
			{
            j1 = j + l;
            x0r = a[j] - a[j1];
            x0i = a[j + 1] - a[j1 + 1];
            a[j] += a[j1];
            a[j + 1] += a[j1 + 1];
            a[j1] = x0r;
            a[j1 + 1] = x0i;
			}
		}

	//n = 2*FFTSIZE 
	n = n>>1;
	//now n = FFTSIZE 

	// FFT output index 0 to N/2-1
	// is frequency output 0 to +Fs/2 Hz  ( 0 Hz DC term ) 
	for( l=0,j=n/2; j<n; l+=2,j++)
		{
		x0r = (a[l]*a[l]) + (a[l+1]*a[l+1]);
		x0i = log10( x0r + m_K_C) + m_K_B;
		//do straight average up until m_AveSize fft's
		//do IIR filter after m_AveSize fft's
		if(m_TotalCount <= m_AveSize) pFFTSumBuf[j] = pFFTSumBuf[j] + x0i;
		else  pFFTSumBuf[j] = pFFTSumBuf[j] - pFFTAveBuf[j] + x0i;
		pFFTAveBuf[j] = pFFTSumBuf[j]/(double)m_AveCount;
		}
	// FFT output index N/2 to N-1  (times 2 since complex samples)
	// is frequency output -Fs/2 to 0  
	for( l=n,j=0; j<n/2; l+=2,j++)
		{
		x0r = (a[l]*a[l]) + (a[l+1]*a[l+1]);
		x0i = log10( x0r + m_K_C) + m_K_B;
		//do straight average up until m_AveSize fft's
		//do IIR filter after m_AveSize fft's
		if(m_TotalCount <= m_AveSize) pFFTSumBuf[j] = pFFTSumBuf[j] + x0i;
		else pFFTSumBuf[j] = pFFTSumBuf[j] - pFFTAveBuf[j] + x0i;
		pFFTAveBuf[j] = pFFTSumBuf[j]/(double)m_AveCount;
		}
}

///////////////////////////////////////////////////////////////////
/* -------- initializing routines -------- */
///////////////////////////////////////////////////////////////////
void MyFFT::makewt(int nw, int *ip, double *w)
{
    int j, nwh;
    double delta, x, y;
    
    ip[0] = nw;
    ip[1] = 1;
    if (nw > 2) 
		{
        nwh = nw >> 1;
        delta = atan(1.0) / nwh;
        w[0] = 1;
        w[1] = 0;
        w[nwh] = cos(delta * nwh);
        w[nwh + 1] = w[nwh];
        if (nwh > 2)
			{
            for (j = 2; j < nwh; j += 2)
				{
                x = cos(delta * j);
                y = sin(delta * j);
                w[j] = x;
                w[j + 1] = y;
                w[nw - j] = y;
                w[nw - j + 1] = x;
				}	
            bitrv2(nw, ip + 2, w);
			}
		}
}

///////////////////////////////////////////////////////////////////
void MyFFT::makect(int nc, int *ip, double *c)
{
    int j, nch;
    double delta;
    
    ip[1] = nc;
    if (nc > 1) {
        nch = nc >> 1;
        delta = atan(1.0) / nch;
        c[0] = cos(delta * nch);
        c[nch] = 0.5 * c[0];
        for (j = 1; j < nch; j++) {
            c[j] = 0.5 * cos(delta * j);
            c[nc - j] = 0.5 * sin(delta * j);
        }
    }
}

///////////////////////////////////////////////////////////////////
/* -------- child routines -------- */
///////////////////////////////////////////////////////////////////
void MyFFT::bitrv2(int n, int *ip, double *a)
{
    int j, j1, k, k1, l, m, m2;
    double xr, xi, yr, yi;
    
    ip[0] = 0;
    l = n;
    m = 1;
    while ((m << 3) < l)
		{
        l >>= 1;
        for (j = 0; j < m; j++) ip[m + j] = ip[j] + l;
        m <<= 1;
		}
    m2 = 2 * m;
    if ((m << 3) == l) 
		{
        for (k = 0; k < m; k++) 
			{
            for (j = 0; j < k; j++)
				{
                j1 = 2 * j + ip[k];
                k1 = 2 * k + ip[j];
                xr = a[j1];
                xi = a[j1 + 1];
                yr = a[k1];
                yi = a[k1 + 1];
                a[j1] = yr;
                a[j1 + 1] = yi;
                a[k1] = xr;
                a[k1 + 1] = xi;
                j1 += m2;
                k1 += 2 * m2;
                xr = a[j1];
                xi = a[j1 + 1];
                yr = a[k1];
                yi = a[k1 + 1];
                a[j1] = yr;
                a[j1 + 1] = yi;
                a[k1] = xr;
                a[k1 + 1] = xi;
                j1 += m2;
                k1 -= m2;
                xr = a[j1];
                xi = a[j1 + 1];
                yr = a[k1];
                yi = a[k1 + 1];
                a[j1] = yr;
                a[j1 + 1] = yi;
                a[k1] = xr;
                a[k1 + 1] = xi;
                j1 += m2;
                k1 += 2 * m2;
                xr = a[j1];
                xi = a[j1 + 1];
                yr = a[k1];
                yi = a[k1 + 1];
                a[j1] = yr;
                a[j1 + 1] = yi;
                a[k1] = xr;
                a[k1 + 1] = xi;
				}
            j1 = 2 * k + m2 + ip[k];
            k1 = j1 + m2;
            xr = a[j1];
            xi = a[j1 + 1];
            yr = a[k1];
            yi = a[k1 + 1];
            a[j1] = yr;
            a[j1 + 1] = yi;
            a[k1] = xr;
            a[k1 + 1] = xi;
			}
		} 
	else 
		{
        for (k = 1; k < m; k++) {
            for (j = 0; j < k; j++) 
				{
                j1 = 2 * j + ip[k];
                k1 = 2 * k + ip[j];
                xr = a[j1];
                xi = a[j1 + 1];
                yr = a[k1];
                yi = a[k1 + 1];
                a[j1] = yr;
                a[j1 + 1] = yi;
                a[k1] = xr;
                a[k1 + 1] = xi;
                j1 += m2;
                k1 += m2;
                xr = a[j1];
                xi = a[j1 + 1];
                yr = a[k1];
                yi = a[k1 + 1];
                a[j1] = yr;
                a[j1 + 1] = yi;
                a[k1] = xr;
                a[k1 + 1] = xi;
				}
			}
		}
}

///////////////////////////////////////////////////////////////////
void MyFFT::cftfsub(int n, double *a, double *w)
{
    int j, j1, j2, j3, l;
    double x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i;
    
    l = 2;
    if (n > 8)
		{
        cft1st(n, a, w);
        l = 8;
        while ((l << 2) < n)
			{
            cftmdl(n, l, a, w);
            l <<= 2;
			}
		}
    if ((l << 2) == n)
		{
        for (j = 0; j < l; j += 2)
			{
            j1 = j + l;
            j2 = j1 + l;
            j3 = j2 + l;
            x0r = a[j] + a[j1];
            x0i = a[j + 1] + a[j1 + 1];
            x1r = a[j] - a[j1];
            x1i = a[j + 1] - a[j1 + 1];
            x2r = a[j2] + a[j3];
            x2i = a[j2 + 1] + a[j3 + 1];
            x3r = a[j2] - a[j3];
            x3i = a[j2 + 1] - a[j3 + 1];
            a[j] = x0r + x2r;
            a[j + 1] = x0i + x2i;
            a[j2] = x0r - x2r;
            a[j2 + 1] = x0i - x2i;
            a[j1] = x1r - x3i;
            a[j1 + 1] = x1i + x3r;
            a[j3] = x1r + x3i;
            a[j3 + 1] = x1i - x3r;
			}
		}
	else
		{
        for (j = 0; j < l; j += 2)
			{
            j1 = j + l;
            x0r = a[j] - a[j1];
            x0i = a[j + 1] - a[j1 + 1];
            a[j] += a[j1];
            a[j + 1] += a[j1 + 1];
            a[j1] = x0r;
            a[j1 + 1] = x0i;
			}
		}
}

///////////////////////////////////////////////////////////////////
void MyFFT::cft1st(int n, double *a, double *w)
{
    int j, k1, k2;
    double wk1r, wk1i, wk2r, wk2i, wk3r, wk3i;
    double x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i;
    
    x0r = a[0] + a[2];
    x0i = a[1] + a[3];
    x1r = a[0] - a[2];
    x1i = a[1] - a[3];
    x2r = a[4] + a[6];
    x2i = a[5] + a[7];
    x3r = a[4] - a[6];
    x3i = a[5] - a[7];
    a[0] = x0r + x2r;
    a[1] = x0i + x2i;
    a[4] = x0r - x2r;
    a[5] = x0i - x2i;
    a[2] = x1r - x3i;
    a[3] = x1i + x3r;
    a[6] = x1r + x3i;
    a[7] = x1i - x3r;
    wk1r = w[2];
    x0r = a[8] + a[10];
    x0i = a[9] + a[11];
    x1r = a[8] - a[10];
    x1i = a[9] - a[11];
    x2r = a[12] + a[14];
    x2i = a[13] + a[15];
    x3r = a[12] - a[14];
    x3i = a[13] - a[15];
    a[8] = x0r + x2r;
    a[9] = x0i + x2i;
    a[12] = x2i - x0i;
    a[13] = x0r - x2r;
    x0r = x1r - x3i;
    x0i = x1i + x3r;
    a[10] = wk1r * (x0r - x0i);
    a[11] = wk1r * (x0r + x0i);
    x0r = x3i + x1r;
    x0i = x3r - x1i;
    a[14] = wk1r * (x0i - x0r);
    a[15] = wk1r * (x0i + x0r);
    k1 = 0;
    for (j = 16; j < n; j += 16) 
		{
        k1 += 2;
        k2 = 2 * k1;
        wk2r = w[k1];
        wk2i = w[k1 + 1];
        wk1r = w[k2];
        wk1i = w[k2 + 1];
        wk3r = wk1r - 2 * wk2i * wk1i;
        wk3i = 2 * wk2i * wk1r - wk1i;
        x0r = a[j] + a[j + 2];
        x0i = a[j + 1] + a[j + 3];
        x1r = a[j] - a[j + 2];
        x1i = a[j + 1] - a[j + 3];
        x2r = a[j + 4] + a[j + 6];
        x2i = a[j + 5] + a[j + 7];
        x3r = a[j + 4] - a[j + 6];
        x3i = a[j + 5] - a[j + 7];
        a[j] = x0r + x2r;
        a[j + 1] = x0i + x2i;
        x0r -= x2r;
        x0i -= x2i;
        a[j + 4] = wk2r * x0r - wk2i * x0i;
        a[j + 5] = wk2r * x0i + wk2i * x0r;
        x0r = x1r - x3i;
        x0i = x1i + x3r;
        a[j + 2] = wk1r * x0r - wk1i * x0i;
        a[j + 3] = wk1r * x0i + wk1i * x0r;
        x0r = x1r + x3i;
        x0i = x1i - x3r;
        a[j + 6] = wk3r * x0r - wk3i * x0i;
        a[j + 7] = wk3r * x0i + wk3i * x0r;
        wk1r = w[k2 + 2];
        wk1i = w[k2 + 3];
        wk3r = wk1r - 2 * wk2r * wk1i;
        wk3i = 2 * wk2r * wk1r - wk1i;
        x0r = a[j + 8] + a[j + 10];
        x0i = a[j + 9] + a[j + 11];
        x1r = a[j + 8] - a[j + 10];
        x1i = a[j + 9] - a[j + 11];
        x2r = a[j + 12] + a[j + 14];
        x2i = a[j + 13] + a[j + 15];
        x3r = a[j + 12] - a[j + 14];
        x3i = a[j + 13] - a[j + 15];
        a[j + 8] = x0r + x2r;
        a[j + 9] = x0i + x2i;
        x0r -= x2r;
        x0i -= x2i;
        a[j + 12] = -wk2i * x0r - wk2r * x0i;
        a[j + 13] = -wk2i * x0i + wk2r * x0r;
        x0r = x1r - x3i;
        x0i = x1i + x3r;
        a[j + 10] = wk1r * x0r - wk1i * x0i;
        a[j + 11] = wk1r * x0i + wk1i * x0r;
        x0r = x1r + x3i;
        x0i = x1i - x3r;
        a[j + 14] = wk3r * x0r - wk3i * x0i;
        a[j + 15] = wk3r * x0i + wk3i * x0r;
		}
}

///////////////////////////////////////////////////////////////////
void MyFFT::cftmdl(int n, int l, double *a, double *w)
{
    int j, j1, j2, j3, k, k1, k2, m, m2;
    double wk1r, wk1i, wk2r, wk2i, wk3r, wk3i;
    double x0r, x0i, x1r, x1i, x2r, x2i, x3r, x3i;
    
    m = l << 2;
    for (j = 0; j < l; j += 2) 
		{
        j1 = j + l;
        j2 = j1 + l;
        j3 = j2 + l;
        x0r = a[j] + a[j1];
        x0i = a[j + 1] + a[j1 + 1];
        x1r = a[j] - a[j1];
        x1i = a[j + 1] - a[j1 + 1];
        x2r = a[j2] + a[j3];
        x2i = a[j2 + 1] + a[j3 + 1];
        x3r = a[j2] - a[j3];
        x3i = a[j2 + 1] - a[j3 + 1];
        a[j] = x0r + x2r;
        a[j + 1] = x0i + x2i;
        a[j2] = x0r - x2r;
        a[j2 + 1] = x0i - x2i;
        a[j1] = x1r - x3i;
        a[j1 + 1] = x1i + x3r;
        a[j3] = x1r + x3i;
        a[j3 + 1] = x1i - x3r;
		}
    wk1r = w[2];
    for (j = m; j < l + m; j += 2) 
		{
        j1 = j + l;
        j2 = j1 + l;
        j3 = j2 + l;
        x0r = a[j] + a[j1];
        x0i = a[j + 1] + a[j1 + 1];
        x1r = a[j] - a[j1];
        x1i = a[j + 1] - a[j1 + 1];
        x2r = a[j2] + a[j3];
        x2i = a[j2 + 1] + a[j3 + 1];
        x3r = a[j2] - a[j3];
        x3i = a[j2 + 1] - a[j3 + 1];
        a[j] = x0r + x2r;
        a[j + 1] = x0i + x2i;
        a[j2] = x2i - x0i;
        a[j2 + 1] = x0r - x2r;
        x0r = x1r - x3i;
        x0i = x1i + x3r;
        a[j1] = wk1r * (x0r - x0i);
        a[j1 + 1] = wk1r * (x0r + x0i);
        x0r = x3i + x1r;
        x0i = x3r - x1i;
        a[j3] = wk1r * (x0i - x0r);
        a[j3 + 1] = wk1r * (x0i + x0r);
		}
    k1 = 0;
    m2 = 2 * m;
    for (k = m2; k < n; k += m2)
		{
        k1 += 2;
        k2 = 2 * k1;
        wk2r = w[k1];
        wk2i = w[k1 + 1];
        wk1r = w[k2];
        wk1i = w[k2 + 1];
        wk3r = wk1r - 2 * wk2i * wk1i;
        wk3i = 2 * wk2i * wk1r - wk1i;
        for (j = k; j < l + k; j += 2)
			{
            j1 = j + l;
            j2 = j1 + l;
            j3 = j2 + l;
            x0r = a[j] + a[j1];
            x0i = a[j + 1] + a[j1 + 1];
            x1r = a[j] - a[j1];
            x1i = a[j + 1] - a[j1 + 1];
            x2r = a[j2] + a[j3];
            x2i = a[j2 + 1] + a[j3 + 1];
            x3r = a[j2] - a[j3];
            x3i = a[j2 + 1] - a[j3 + 1];
            a[j] = x0r + x2r;
            a[j + 1] = x0i + x2i;
            x0r -= x2r;
            x0i -= x2i;
            a[j2] = wk2r * x0r - wk2i * x0i;
            a[j2 + 1] = wk2r * x0i + wk2i * x0r;
            x0r = x1r - x3i;
            x0i = x1i + x3r;
            a[j1] = wk1r * x0r - wk1i * x0i;
            a[j1 + 1] = wk1r * x0i + wk1i * x0r;
            x0r = x1r + x3i;
            x0i = x1i - x3r;
            a[j3] = wk3r * x0r - wk3i * x0i;
            a[j3 + 1] = wk3r * x0i + wk3i * x0r;
			}
        wk1r = w[k2 + 2];
        wk1i = w[k2 + 3];
        wk3r = wk1r - 2 * wk2r * wk1i;
        wk3i = 2 * wk2r * wk1r - wk1i;
        for (j = k + m; j < l + (k + m); j += 2)
			{
            j1 = j + l;
            j2 = j1 + l;
            j3 = j2 + l;
            x0r = a[j] + a[j1];
            x0i = a[j + 1] + a[j1 + 1];
            x1r = a[j] - a[j1];
            x1i = a[j + 1] - a[j1 + 1];
            x2r = a[j2] + a[j3];
            x2i = a[j2 + 1] + a[j3 + 1];
            x3r = a[j2] - a[j3];
            x3i = a[j2 + 1] - a[j3 + 1];
            a[j] = x0r + x2r;
            a[j + 1] = x0i + x2i;
            x0r -= x2r;
            x0i -= x2i;
            a[j2] = -wk2i * x0r - wk2r * x0i;
            a[j2 + 1] = -wk2i * x0i + wk2r * x0r;
            x0r = x1r - x3i;
            x0i = x1i + x3r;
            a[j1] = wk1r * x0r - wk1i * x0i;
            a[j1 + 1] = wk1r * x0i + wk1i * x0r;
            x0r = x1r + x3i;
            x0i = x1i - x3r;
            a[j3] = wk3r * x0r - wk3i * x0i;
            a[j3 + 1] = wk3r * x0i + wk3i * x0r;
	        }
		}
}






